
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
var stringService = Components.classes["@mozilla.org/intl/stringbundle;1"];
var stringBundle = stringService.createBundle("chrome://prefswitch/locale/global.properties");
var stringStatusBar = stringBundle.GetStringFromName("status-bar");
*/

const ID_STATUS_BAR = "status-bar";
const ID_POPUPSET = "prefswitch-popupset";
const ID_MENUPOPUP = "prefswitch-menupopup";
const ID_PROFILE_MENUPOPUP = "prefswitch-profile-menupopup";
const ID_PROFILE_SEPARATOR = "prefswitch-profile-separator";
const ID_SHOWHIDE_MENUPOPUP = "prefswitch-showHide-menupopup";
const ID_CSS_POPUP = "prefswitch-cssPopup";
const ID_JS_POPUP = "prefswitch-jsPopup";
const ID_SEPARATOR = "prefswitch-separator";
const CLASS_INITIAL = "prefswitch-initial";
const CLASS_INJECT = "prefswitch-inject";

const INJECT_PATH = PREF_PATH + "/inject";

function Overlay()
{
	this.appcontent = null;
	this.control = null;
	this.statusbar = null;
	this.popupset = null;
	this.profileMenupopup = null;
	this.profileSeparator = null;
	this.cssPopup = null;
	this.jsPopup = null;
	this.separator = null;
	this.observer = null;
	this.jsObject = {};

	this.openDialog = function(dialogName)
	{
		var url = null;
		var name = null;
		var features = null;
		switch(dialogName)
		{
			case "profile":
				url = "chrome://prefswitch/content/profiledialog.xul";
				name = "profiledialog";
				features = "modal";
				break;
			case "switch":
				url = "chrome://prefswitch/content/switchdialog.xul";
				name = "switchdialog";
				features = "";
				break;
			default: return;
		}
		window.openDialog(url, name, features, this.control);
	};

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.control = new Control(this);
				this.appcontent = $("appcontent");
				this.appcontent.addEventListener("DOMContentLoaded", this, false);
				this.statusbar = $(ID_STATUS_BAR);
				this.popupset = $(ID_POPUPSET);
				this.profileMenupopup = $(ID_PROFILE_MENUPOPUP);
				this.profileSeparator = $(ID_PROFILE_SEPARATOR);
				this.showHideMenupopup = $(ID_SHOWHIDE_MENUPOPUP);
				this.cssPopup = $(ID_CSS_POPUP);
				this.jsPopup = $(ID_JS_POPUP);
				this.separator = $(ID_SEPARATOR);
				this.updateAll();
				this.observer = new Observer(this.control, UPDATE_ALL);
				this.observer.register();
				break;
/*			case "unload":
				this.appcontent.removeEventListener("DOMContentLoaded", this.updateContentLoaded, false);
				var statusbarpanel = this.statusbar.firstChild;
				while (statusbarpanel)
				{
					if (statusbarpanel.getAttribute("class") == CLASS_INITIAL) statusbarpanel.o.unregister();
					statusbarpanel = statusbarpanel.nextSibling;
				}
				break;*/
			case "DOMContentLoaded":
				for (var key in this.jsObject)
				{
					if (!this.jsObject[key]) continue;
					this.injectJavaScript(this.jsObject[key], event.target.defaultView);
				}
		}
	};

	this.updateAll = function()
	{
		var profile = this.updateProfiles();

		this.removePopups();
		this.removeStatusbarpanels();
		this.removeMenuitems(this.cssPopup);
		this.removeMenuitems(this.jsPopup);

		this.addStatusbarpanels();
		this.addMenuitems();

		if (profile) this.loadProfile(profile);
	};

	this.removeStatusbarpanels = function()
	{
		var element = this.separator.previousSibling;
		while (element.getAttribute("class") == CLASS_INITIAL)
		{
			var next = element.previousSibling;
			element.observer.unregister();
			this.statusbar.removeChild(element);
			element = next;
		}
	};

	this.removePopups = function()
	{
		var popup = this.popupset.firstChild;
		while (popup)
		{
			var next = popup.nextSibling;
			if (popup.tagName == "popup") this.popupset.removeChild(popup);
			popup = next;
		}
	};

	this.removeMenuitems = function(element)
	{
		var menuitem = element.firstChild;
		while (menuitem)
		{
			var next = menuitem.nextSibling;
			menuitem.observer.unregister();
			element.removeChild(menuitem);
			menuitem = next;
		}
	};

	this.addStatusbarpanels = function()
	{
		while (this.showHideMenupopup.firstChild) this.showHideMenupopup.removeChild(this.showHideMenupopup.firstChild);
		var aSwitch = this.control.getSwitches();

		// CREATE STATUSBARPANELS
		while (aSwitch.length > 0)
		{
			var mySwitch = aSwitch.shift();
			var statusbarpanel = this.createSwitchElement(mySwitch, "statusbarpanel", CLASS_INITIAL, mySwitch.getInitial(), "onclick");
			this.statusbar.insertBefore(statusbarpanel, this.separator);

			// CREATE SHOW-HIDE-MENUITEMS
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("type", "checkbox");
			menuitem.setAttribute("label", mySwitch.getName());
			menuitem.setAttribute("checked", !statusbarpanel.getAttribute("hidden"));
			statusbarpanel.showHideMenuitem = menuitem;
			menuitem.statusbarpanel = statusbarpanel;
			menuitem.setAttribute("oncommand", "this.statusbarpanel.setAttribute('hidden', !this.getAttribute('checked'));");
			this.showHideMenupopup.appendChild(menuitem);
		}
	};

	this.addMenuitems = function()
	{
		// GET FILES
		var directory = this.control.getDirectory(INJECT_PATH);
		var entries = directory.directoryEntries;
		var aFiles = [];
		while (entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isFile()) aFiles.push(entry);
		}

		// CREATE SWITCHES
		var aSwitches = [];
		while (aFiles.length > 0)
		{
			var file = aFiles.shift();
			var mySwitch = new Switch(file.leafName, file.leafName, true, file.leafName);
			mySwitch.setInject(true);
			mySwitch.setInjectPath(file.path);
			var disabledState = new State("Extract " + file.leafName, "Extract " + file.leafName, "", "", "lightgrey");
			disabledState.addPreference(new Preference(ROOT + file.leafName, BOOLEAN, false, "Extract " + file.leafName));
			mySwitch.addState(disabledState);
			var enabledState = new State("Inject " + file.leafName, "Inject " + file.leafName, "", "", "black");
			enabledState.addPreference(new Preference(ROOT + file.leafName, BOOLEAN, true, "Inject " + file.leafName));
			mySwitch.addState(enabledState);
			aSwitches.push(mySwitch);
		}

		// CREATE MENUITEMS
		while (aSwitches.length > 0)
		{
			var mySwitch = aSwitches.shift();
			var menuitem = this.createSwitchElement(mySwitch, "menuitem", "", mySwitch.getName(), "oncommand");
			menuitem.setAttribute("type", "checkbox");
			switch(mySwitch.getInjectPathExtension())
			{
				case "css": this.cssPopup.appendChild(menuitem); break;
				case "js": this.jsPopup.appendChild(menuitem);
			}
		}
	};

	this.createSwitchElement = function(mySwitch, elementType, className, labelName, eventName)
	{
		var element = document.createElement(elementType);
		element.mySwitch = mySwitch;
		element.observer = new Observer(this.control, UPDATE_ONE, ROOT, mySwitch.getID(), element);
		element.observer.register();
		element.setAttribute("class", className);
		element.setAttribute("label", labelName);
		element.setAttribute("context", ID_MENUPOPUP);

		// APPEND CLICK EVENT OR CREATE POPUP MENU
		if (mySwitch.isBooleanSwitch()) element.setAttribute(eventName, "overlay.onclick(event, this.mySwitch, this.mySwitch.getState(0), true)");
		else
		{
			var popup = document.createElement("popup");
			var id = "prefswitch-popup-" + mySwitch.getID();
			popup.setAttribute("id", id);

			// ADD MENUITEMS
			for (var i = 0; i < mySwitch.getStates().length; i++)
			{
				var state = mySwitch.getState(i);
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("type", "radio");
				menuitem.setAttribute("name", "prefswitch-radio-" + mySwitch.getID());
				menuitem.setAttribute("label", state.getDescription());
				menuitem.mySwitch = mySwitch;
				menuitem.state = state;
				state.menuitem = menuitem;
				menuitem.setAttribute("oncommand", "overlay.onclick(event, this.mySwitch, this.state, false)");
				popup.appendChild(menuitem);
			}
			this.popupset.appendChild(popup);
			element.setAttribute("popup", id);
		}
		this.updateOne(element);
		return element;
	};

	this.updateOne = function(element)
	{
		var mySwitch = element.mySwitch;
		var value = this.control.getValue(ROOT + mySwitch.getID(), mySwitch.getState(0).getID(), STRING);
		var state = mySwitch.getStateByID(value);
		switch(element.tagName)
		{
			case "statusbarpanel":
				var color = state.getColor();
				if (color != "") element.setAttribute("style", "color:" + color);
				var tooltiptext = state.getTooltip();
				if (tooltiptext != "") element.setAttribute("tooltiptext", tooltiptext);
				if (state.menuitem)
				{
					var menuitem = state.menuitem.parentNode.firstChild;
					while (menuitem)
					{
						if (state.menuitem == menuitem) menuitem.setAttribute("checked", "true");
						else menuitem.setAttribute("checked", "false");
						menuitem = menuitem.nextSibling;
					}
				}
				break;
			case "menuitem":
				var path = mySwitch.getInjectPath();
				var inject = state.getPreference(0).getValue() == "true";
				switch(mySwitch.getInjectPathExtension())
				{
					case "css": this.injectCSS(path, inject); break;
					case "js": this.setJavascript(mySwitch.getID(), path, inject); break;
				}
				element.setAttribute("checked", "" + inject);
				break;
		}
	};

	this.injectCSS = function(path, value)
	{
		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("file://" + path, CHARSET, null); // URI, CHARSET, BASEURI
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
		if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	};

	this.setJavascript = function(key, path, value)
	{
		if (value) this.jsObject[key] = path;
		else this.jsObject[key] = null;
	};

	this.injectJavaScript = function(path, defaultView)
	{
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		var data = this.control.getData(file);
		var safeView = new XPCNativeWrapper(defaultView);
		var sandbox = Components.utils.Sandbox(safeView);
		sandbox.window = safeView;
		sandbox.document = safeView.document;
		sandbox.XPathResult = Components.interfaces.nsIDOMXPathResult;
		sandbox.__proto__ = safeView;
		Components.utils.evalInSandbox(data, sandbox);
	};

	this.onclick = function(event, mySwitch, state, isBooleanSwitch)
	{
		if (event && event.button && event.button > 0) return;
		var switchID = mySwitch.getID();
		if (isBooleanSwitch)
		{
			var firstStateID = mySwitch.getState(0).getID();
			var value = this.control.getValue(ROOT + switchID, firstStateID, STRING);
			if (value == firstStateID) state = mySwitch.getState(1);
			var preference = state.getPreference(0);
			if (preference.getName().indexOf(ROOT) == -1)
				this.control.setValue(preference.getName(), preference.getValue() == "true", preference.getType());
		}
		else
		{
			for (var i = 0; i < state.getPreferences().length; i++)
			{
				var preference = state.getPreference(i);
				this.control.setValue(preference.getName(), preference.getValue(), preference.getType());
			}
		}
		this.control.setValue(ROOT + switchID, state.getID(), STRING);
	};

	this.updateProfiles = function()
	{
		while (this.profileMenupopup.firstChild.getAttribute("id") != ID_PROFILE_SEPARATOR) this.profileMenupopup.removeChild(this.profileMenupopup.firstChild);
		var profileID = this.control.getValue(ROOT + "profileID", null, STRING);
		var currentProfile = null;
		var aProfile = this.control.getProfiles();
		while (aProfile.length > 0)
		{
			var profile = aProfile.shift();
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("type", "radio");
			menuitem.setAttribute("name", "prefswitch-profile-menuitem");
			menuitem.setAttribute("label", profile.getName());
			menuitem.profile = profile;
			menuitem.setAttribute("oncommand", "overlay.loadProfile(this.profile)");
			if (profileID == profile.getID())
			{
				menuitem.setAttribute("checked", "true");
				currentProfile = profile;
			}
			this.profileMenupopup.insertBefore(menuitem, this.profileSeparator);
		}
		return currentProfile;
	};

	this.loadProfile = function(profile)
	{
		var statusbarpanel = this.statusbar.firstChild;
		while (statusbarpanel)
		{
			switch(statusbarpanel.getAttribute("class"))
			{
				case CLASS_INITIAL:
					var mySwitch = statusbarpanel.mySwitch;
					var feature = profile.getFeature(mySwitch.getID())
					var state = mySwitch.getStateByID(feature.value);

					statusbarpanel.setAttribute("hidden", "" + !feature.show);
					statusbarpanel.showHideMenuitem.setAttribute("checked", "" + feature.show);

					this.onclick(LOAD_PROFILE, mySwitch, state, false);
			}
			statusbarpanel = statusbarpanel.nextSibling;
		}
		this.control.setValue(ROOT + "profileID", profile.getID(), STRING);
	};
}

function $(id) {return document.getElementById(id);}

var overlay = new Overlay();
window.addEventListener("load", overlay, false);
//window.addEventListener("unload", overlay, false);
